//
//  BFVideoThumbImageFetchHelper.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2021/12/3.
//

import Foundation
import AVFoundation
import BFCommonKit

/// 视频分解成帧
/// - parameter fileUrl                 : 视频地址
/// - parameter fps                     : 自定义帧数 每秒内取的帧数
/// - parameter splitCompleteClosure    : 回调
func splitVideoFileUrlFps(urlAsset:AVURLAsset, fps:Float, splitCompleteClosure:@escaping ((Bool, [UIImage]?) -> Void)) {
    var splitImages = [UIImage]()
    
    var times = [NSValue]()
    
    for i in 0...Int(urlAsset.duration.seconds * Float64(fps)) {
        let timeValue = NSValue(time: CMTimeMake(value: Int64(i), timescale: Int32(fps)) )
        
        times.append(timeValue)
    }
    
    let imgGenerator = AVAssetImageGenerator(asset: urlAsset)
    imgGenerator.requestedTimeToleranceBefore = CMTime.zero
    imgGenerator.requestedTimeToleranceAfter = CMTime.zero
    
    let timesCount = times.count
    var cocu = 0
    //获取每一帧的图片
    imgGenerator.generateCGImagesAsynchronously(forTimes: times) { (requestedTime, image, actualTime, result, error) in
        cocu += 1
        var isSuccess = false
        switch (result) {
        case AVAssetImageGenerator.Result.cancelled:
            BFLog(1, message: "aaa: cancel")

        case AVAssetImageGenerator.Result.failed:
            BFLog(1, message: "aaa: failed")

        case AVAssetImageGenerator.Result.succeeded:
            let framImg = UIImage(cgImage: image!)
            splitImages.append(framImg)
            BFLog(1, message: "aaa: \(requestedTime.seconds) - \(actualTime.seconds)")
//            if (Int(requestedTime.value) == (timesCount-1)) {
//            }
            isSuccess = true
        @unknown default:
            break
        }
        if cocu == timesCount { //最后一帧时 回调赋值
            splitCompleteClosure(isSuccess, splitImages)
            BFLog(1, message: "aaa: complete")

        }
    }
   
}

/// 视频分解成帧
/// - parameter fileUrl                 : 视频地址
/// - parameter fps                     : 自定义帧数 每秒内取的帧数
/// - parameter splitCompleteClosure    : 回调
func getThumbImageAtTime(urlAsset: AVURLAsset, time:CMTime) -> UIImage? {
    let imgGenerator = AVAssetImageGenerator(asset: urlAsset)
    imgGenerator.requestedTimeToleranceBefore = CMTime.zero
    imgGenerator.requestedTimeToleranceAfter = CMTime.zero
    
    var cgImg = try? imgGenerator.copyCGImage(at: time, actualTime: nil)
    if cgImg == nil  {
        imgGenerator.requestedTimeToleranceBefore = CMTime.positiveInfinity
        imgGenerator.requestedTimeToleranceAfter = CMTime.positiveInfinity
        cgImg = try? imgGenerator.copyCGImage(at: time, actualTime: nil)
    }

    return cgImg == nil ? nil : UIImage(cgImage: cgImg!)
}
